@@ -1,9 +1,28 @@
 # -*- coding: utf-8 -*-
 
-__version__ = '0.4.5'
+"""
+fritzhosts.py
 
-import argparse
-import fritzconnection
+Utility modul for FritzConnection to list the known hosts.
+
+Older versions of FritzOS lists only up to 16 entries.
+For newer versions this limitation is gone.
+
+License: MIT https://opensource.org/licenses/MIT
+Source: https://github.com/kbr/fritzconnection
+Author: Klaus Bremer
+"""
+
+import os, argparse
+
+# tiny hack to run this as a package but also from the command line. In
+# the latter case ValueError is raised from python 2.7,
+# SystemError from Python 3.5
+# ImportError from Python 3.6
+try:
+    from . import fritzconnection
+except (ValueError, SystemError, ImportError):
+    import fritzconnection
 
 
 SERVICE = 'Hosts'
@@ -11,17 +30,12 @@ SERVICE = 'Hosts'
 
 # version-access:
 def get_version():
-    return __version__
+    return fritzconnection.get_version()
 
 
 class FritzHosts(object):
 
-    def __init__(self,
-                 fc=None,
-                 address=fritzconnection.FRITZ_IP_ADDRESS,
-                 port=fritzconnection.FRITZ_TCP_PORT,
-                 user=fritzconnection.FRITZ_USERNAME,
-                 password=''):
+    def __init__(self, fc=None, address=None, port=None, user=None, password=None):
         super(FritzHosts, self).__init__()
         if fc is None:
             fc = fritzconnection.FritzConnection(address, port, user, password)
@@ -53,15 +67,13 @@ class FritzHosts(object):
         The dict-keys are: 'ip', 'name', 'mac', 'status'
         """
         result = []
-        index = 0
-        while index < self.host_numbers:
+        for index in range(self.host_numbers):
             host = self.get_generic_host_entry(index)
             result.append({
                 'ip': host['NewIPAddress'],
                 'name': host['NewHostName'],
                 'mac': host['NewMACAddress'],
                 'status': host['NewActive']})
-            index += 1
         return result
 
 
@@ -82,29 +94,33 @@ def print_hosts(fh):
         'n', 'ip', 'name', 'mac', 'status'))
     hosts = fh.get_hosts_info()
     for index, host in enumerate(hosts):
-        if host['status'] == '1':
-            status = 'active'
-        else:
-            status = '-'
+        status = 'active' if host['status'] == '1' else  '-'
+        ip = '-' if host['ip'] == None else host['ip']
+        mac = '-' if host['mac'] == None else host['mac']
         print('{:>3}: {:<15} {:<26} {:<17}   {}'.format(
             index,
-            host['ip'],
+            ip,
             host['name'],
-            host['mac'],
+            mac,
             status,
             )
         )
     print('\n')
 
 
-def _print_detail(fh, detail):
-    mac_address = detail[0]
-    print('\n{:<23}{}\n'.format('Details for host:', mac_address))
+def _print_detail(fh, detail, quiet):
+    mac_address = detail[0].upper()
     info = fh.get_specific_host_entry(mac_address)
-    for key, value in info.items():
-        print('{:<23}: {}'.format(key, value))
-    print('\n')
-
+    if info:
+        if not quiet:
+            print('\n{:<23}{}\n'.format('Details for host:', mac_address))
+            for key, value in info.items():
+                print('{:<23}: {}'.format(key, value))
+            print('\n')
+        else:
+            print(info['NewActive'])
+    else:
+        print('0')
 
 def _print_nums(fh):
     print('{:<20}{}\n'.format('Number of hosts:', fh.host_numbers))
@@ -117,20 +133,20 @@ def _print_nums(fh):
 def _get_cli_arguments():
     parser = argparse.ArgumentParser(description='FritzBox Hosts')
     parser.add_argument('-i', '--ip-address',
-                        nargs='?', default=fritzconnection.FRITZ_IP_ADDRESS,
+                        nargs='?', default=None, const=None,
                         dest='address',
                         help='ip-address of the FritzBox to connect to. '
                              'Default: %s' % fritzconnection.FRITZ_IP_ADDRESS)
     parser.add_argument('--port',
-                        nargs='?', default=fritzconnection.FRITZ_TCP_PORT,
+                        nargs='?', default=None, const=None,
                         dest='port',
                         help='port of the FritzBox to connect to. '
                              'Default: %s' % fritzconnection.FRITZ_TCP_PORT)
     parser.add_argument('-u', '--username',
-                        nargs=1, default=fritzconnection.FRITZ_USERNAME,
+                        nargs='?', default=None, const=None,
                         help='Fritzbox authentication username')
     parser.add_argument('-p', '--password',
-                        nargs=1, default='',
+                        nargs='?', default=None, const=None,
                         help='Fritzbox authentication password')
     parser.add_argument('-a', '--all',
                         action='store_true',
@@ -143,6 +159,10 @@ def _get_cli_arguments():
                         nargs=1, default='',
                         help='Show information about a specific host '
                              '(DETAIL: MAC Address)')
+    parser.add_argument('-q', '--quiet',
+                        action='store_true',
+                        help='Quiet mode '
+                             '(just return state as 0|1 for requested mac address)')
     args = parser.parse_args()
     return args
 
@@ -152,14 +172,17 @@ def _print_status(arguments):
                     port=arguments.port,
                     user=arguments.username,
                     password=arguments.password)
-    _print_header(fh)
+    if not arguments.quiet:
+        _print_header(fh)
     if arguments.detail:
-        _print_detail(fh, arguments.detail)
+        _print_detail(fh, arguments.detail, arguments.quiet)
     elif arguments.nums:
         _print_nums(fh)
     else:
         print_hosts(fh)
 
+def main():
+    _print_status(_get_cli_arguments())
 
 if __name__ == '__main__':
-    _print_status(_get_cli_arguments())
+    main()
