@@ -7,9 +7,11 @@ fritzmonitor.py
 Implement a tkinter-based grafic interface to view basic status- and
 traffic-informations.
 
+License: MIT https://opensource.org/licenses/MIT
+Source: https://github.com/kbr/fritzconnection
+Author: Klaus Bremer
 """
 
-__version__ = '0.1.0'
 
 import argparse
 try:
@@ -20,9 +22,19 @@ except ImportError:
     # python 3
     import tkinter as tk
     import tkinter.font as tkfont
-import fritzconnection
-import fritzstatus
-import fritztools
+try:
+    from . import fritzconnection
+    from . import fritzstatus
+    from . import fritztools
+except (ValueError, SystemError, ImportError):
+    import fritzconnection
+    import fritzstatus
+    import fritztools
+
+
+# version-access:
+def get_version():
+    return fritzconnection.get_version()
 
 
 class MeterRectangle(object):
@@ -193,8 +205,11 @@ def _get_cli_arguments():
     args = parser.parse_args()
     return args
 
-if __name__ == '__main__':
+def main():
     arguments = _get_cli_arguments()
     app = FritzMonitor(address=arguments.address, port=arguments.port)
     app.master.title('FritzMonitor')
     app.mainloop()
+
+if __name__ == '__main__':
+    main()
