@@ -5,37 +5,55 @@
 fritzstatus.py
 
 Modul to read status-informations from an AVM FritzBox.
-"""
 
-__version__ = '0.4.5'
+License: MIT https://opensource.org/licenses/MIT
+Source: https://github.com/kbr/fritzconnection
+Author: Klaus Bremer
+"""
 
 import argparse
 import collections
+import os
 import time
-import fritzconnection
-import fritztools
+
+# tiny hack to run this as a package but also from the command line. In
+# the latter case ValueError is raised from python 2.7 and SystemError
+# from Python 3.5 and ImportError from Python 3.6
+try:
+    from . import fritzconnection
+    from . import fritztools
+except (ValueError, SystemError, ImportError):
+    import fritzconnection
+    import fritztools
 
 
 # version-access:
 def get_version():
-    return __version__
+    return fritzconnection.get_version()
 
 
 class FritzStatus(object):
     """
-    Class for requsting status-informations:
+    Class for requesting status-informations:
     up, down, ip, activity (bytes per second send/received)
     Every property will raise an IOError if the connection
     with the FritzBox fails.
+
+    Keep in mind, that FritzBoxes may return different informations
+    about the status depending whether this service gets called with or
+    without a password.
     """
 
-    def __init__(self,
-                 fc=None,
-                 address=fritzconnection.FRITZ_IP_ADDRESS,
-                 port=fritzconnection.FRITZ_TCP_PORT):
+    def __init__(self, fc=None, address=None, port=None,
+                       user=None, password=None):
         super(FritzStatus, self).__init__()
         if fc is None:
-            fc = fritzconnection.FritzConnection(address=address, port=port)
+            fc = fritzconnection.FritzConnection(
+                address=address,
+                port=port,
+                user=user,
+                password=password,
+            )
         self.fc = fc
         self.last_bytes_sent = self.bytes_sent
         self.last_bytes_received = self.bytes_received
@@ -48,7 +66,7 @@ class FritzStatus(object):
     @property
     def is_linked(self):
         """Returns True if the FritzBox is physically linked to the provider."""
-        status = self.fc.call_action('WANCommonInterfaceConfig',
+        status = self.fc.call_action('WANCommonIFC',
                                      'GetCommonLinkProperties')
         return status['NewPhysicalLinkStatus'] == 'Up'
 
@@ -57,25 +75,31 @@ class FritzStatus(object):
         """
         Returns True if the FritzBox has established an internet-connection.
         """
-        status = self.fc.call_action('WANIPConnection', 'GetStatusInfo')
+        status = self.fc.call_action('WANIPConn', 'GetStatusInfo')
         return status['NewConnectionStatus'] == 'Connected'
 
     @property
     def wan_access_type(self):
         """Returns connection-type: DSL, Cable."""
-        return self.fc.call_action('WANCommonInterfaceConfig',
+        return self.fc.call_action('WANCommonIFC',
             'GetCommonLinkProperties')['NewWANAccessType']
 
     @property
     def external_ip(self):
         """Returns the external ip-address."""
-        return self.fc.call_action('WANIPConnection',
+        return self.fc.call_action('WANIPConn',
             'GetExternalIPAddress')['NewExternalIPAddress']
 
+    @property
+    def external_ipv6(self):
+        """Returns the external ip-address."""
+        return self.fc.call_action('WANIPConn',
+            'X_AVM_DE_GetExternalIPv6Address')['NewExternalIPv6Address']
+
     @property
     def uptime(self):
         """uptime in seconds."""
-        status = self.fc.call_action('WANIPConnection', 'GetStatusInfo')
+        status = self.fc.call_action('WANIPConn', 'GetStatusInfo')
         return status['NewUptime']
 
     @property
@@ -87,13 +111,13 @@ class FritzStatus(object):
 
     @property
     def bytes_sent(self):
-        status = self.fc.call_action('WANCommonInterfaceConfig',
+        status = self.fc.call_action('WANCommonIFC',
                                      'GetTotalBytesSent')
         return status['NewTotalBytesSent']
 
     @property
     def bytes_received(self):
-        status = self.fc.call_action('WANCommonInterfaceConfig',
+        status = self.fc.call_action('WANCommonIFC',
                                      'GetTotalBytesReceived')
         return status['NewTotalBytesReceived']
 
@@ -123,14 +147,28 @@ class FritzStatus(object):
             fritztools.format_num(downstream)
         )
 
+    @property
+    def max_linked_bit_rate(self):
+        """
+        Returns a tuple with the maximun upstream- and downstream-rate
+        of the physical link. The rate is given in bits/sec.
+        """
+        return self._get_max_bit_rate('WANCommonInterfaceConfig')
+
     @property
     def max_bit_rate(self):
         """
         Returns a tuple with the maximun upstream- and downstream-rate
         of the given connection. The rate is given in bits/sec.
         """
-        status = self.fc.call_action('WANCommonInterfaceConfig',
-                                     'GetCommonLinkProperties')
+        return self._get_max_bit_rate('WANCommonIFC')
+
+    def _get_max_bit_rate(self, servicename):
+        """
+        internal method to get the upstream and downstream-rates for
+        different services of the WANCommonInterfaceConfig1 ServiceType.
+        """
+        status = self.fc.call_action(servicename, 'GetCommonLinkProperties')
         downstream = status['NewLayer1DownstreamMaxBitRate']
         upstream = status['NewLayer1UpstreamMaxBitRate']
         return upstream, downstream
@@ -143,6 +181,18 @@ class FritzStatus(object):
         upstream, downstream = self.max_bit_rate
         return upstream / 8.0, downstream / 8.0
 
+    @property
+    def str_max_linked_bit_rate(self):
+        """
+        Returns a human readable maximun upstream- and downstream-rate
+        of the given connection. The rate is given in bits/sec.
+        """
+        upstream, downstream = self.max_linked_bit_rate
+        return (
+            fritztools.format_rate(upstream, unit='bits'),
+            fritztools.format_rate(downstream, unit ='bits')
+        )
+
     @property
     def str_max_bit_rate(self):
         """
@@ -164,12 +214,11 @@ class FritzStatus(object):
 # terminal-output:
 # ---------------------------------------------------------
 
-def print_status(address=fritzconnection.FRITZ_IP_ADDRESS,
-                 port=fritzconnection.FRITZ_TCP_PORT):
+def print_status(address=None, port=None, user=None, password=None):
     print('\nFritzStatus:')
-    print('{:<20}{}'.format('version:', get_version()))
-    fs = FritzStatus(address=address, port=port)
-    for status, info in collections.OrderedDict([
+    print('{:<22}{}'.format('version:', get_version()))
+    fs = FritzStatus(address=address, port=port, user=user, password=password)
+    status_informations = collections.OrderedDict([
         ('model:', fs.modelname),
         ('is linked:', fs.is_linked),
         ('is connected:', fs.is_connected),
@@ -177,9 +226,17 @@ def print_status(address=fritzconnection.FRITZ_IP_ADDRESS,
         ('uptime:', fs.str_uptime),
         ('bytes send:', fs.bytes_sent),
         ('bytes received:', fs.bytes_received),
-        ('max. bit rate:', fs.str_max_bit_rate)
-        ]).items():
-        print('{:<20}{}'.format(status, info))
+        ('max. bit rate:', fs.str_max_bit_rate),
+        ])
+    try:
+        information = fs.str_max_linked_bit_rate
+    except fritzconnection.ServiceError:
+        information = 'password required for information'
+    except fritzconnection.AuthorizationError as err:
+        information = str(err)
+    status_informations['max. linked bit rate:'] = information
+    for status, information in status_informations.items():
+        print('{:<22}{}'.format(status, information))
 
 
 # ---------------------------------------------------------
@@ -189,12 +246,18 @@ def print_status(address=fritzconnection.FRITZ_IP_ADDRESS,
 def _get_cli_arguments():
     parser = argparse.ArgumentParser(description='FritzBox Status')
     parser.add_argument('-i', '--ip-address',
-                        nargs='?', default=fritzconnection.FRITZ_IP_ADDRESS,
+                        nargs='?', default=None, const=None,
                         dest='address',
                         help='ip-address of the FritzBox to connect to. '
                              'Default: %s' % fritzconnection.FRITZ_IP_ADDRESS)
+    parser.add_argument('-u', '--username',
+                        nargs='?', default=None, const=None,
+                        help='Fritzbox authentication username')
+    parser.add_argument('-p', '--password',
+                        nargs='?', default=None, const=None,
+                        help='Fritzbox authentication password')
     parser.add_argument('--port',
-                        nargs='?', default=fritzconnection.FRITZ_TCP_PORT,
+                        nargs='?', default=None, const=None,
                         dest='port',
                         help='port of the FritzBox to connect to. '
                              'Default: %s' % fritzconnection.FRITZ_TCP_PORT)
@@ -203,8 +266,15 @@ def _get_cli_arguments():
 
 
 def _print_status(arguments):
-    print_status(address=arguments.address, port=arguments.port)
+    print_status(
+        address=arguments.address,
+        port=arguments.port,
+        user=arguments.username,
+        password=arguments.password,
+    )
 
+def main():
+    _print_status(_get_cli_arguments())
 
 if __name__ == '__main__':
-    _print_status(_get_cli_arguments())
+    main()
